import json
import logging

from accqsure.exceptions import SpecificationError


class Inspections(object):
    def __init__(self, accqsure):
        self.accqsure = accqsure

    async def get(self, id, **kwargs):
        resp = await self.accqsure._query(f"/inspection/{id}", "GET", kwargs)
        return Inspection(self.accqsure, **resp)

    async def list(self, type, **kwargs):
        resp = await self.accqsure._query(
            f"/inspection",
            "GET",
            dict(type=type, **kwargs),
        )
        inspections = [
            Inspection(self.accqsure, **inspection) for inspection in resp
        ]
        return inspections

    async def create(
        self,
        type,
        name,
        document_type_id,
        manifests,
        draft=None,
        documents=None,
        **kwargs,
    ):

        data = dict(
            name=name,
            type=type,
            document_type_id=document_type_id,
            manifests=manifests,
            draft=draft,
            documents=documents**kwargs,
        )
        payload = {k: v for k, v in data.items() if v is not None}
        logging.info(f"Creating Inspection {name}")
        resp = await self.accqsure._query("/inspection", "POST", None, payload)
        inspection = Inspection(self.accqsure, **resp)
        logging.info(f"Created Inspection {name} with id {inspection.id}")

        return inspection

    async def remove(self, id, **kwargs):
        await self.accqsure._query(
            f"/inspection/{id}", "DELETE", dict(**kwargs)
        )


class Inspection:
    def __init__(self, accqsure, **kwargs):
        self.accqsure = accqsure
        self._entity = kwargs
        self._id = self._entity.get("entity_id")
        self._name = self._entity.get("name")
        self._status = self._entity.get("status")
        self._content_id = self._entity.get("doc_content_id")

    @property
    def id(self) -> str:
        return self._id

    @property
    def status(self) -> str:
        return self._status

    @property
    def name(self) -> str:
        return self._name

    def __str__(self):
        return json.dumps({k: v for k, v in self._entity.items()})

    def __repr__(self):
        return f"Inspection( accqsure , **{self._entity.__repr__()})"

    def __bool__(self):
        return bool(self._id)

    async def remove(self):
        await self.accqsure._query(
            f"/inspection/{self._id}",
            "DELETE",
        )

    async def rename(self, name):
        resp = await self.accqsure._query(
            f"/inspection/{self._id}",
            "PUT",
            None,
            dict(name=name),
        )
        self.__init__(self.accqsure, **resp)
        return self

    async def refresh(self):
        resp = await self.accqsure._query(
            f"/inspection/{self.id}",
            "GET",
        )
        self.__init__(self.accqsure, **resp)
        return self

    async def get_contents(self):
        if not self._content_id:
            raise SpecificationError(
                "content_id", "Content not uploaded for inspection"
            )
        resp = await self.accqsure._query(
            f"/inspection/{self.id}/asset/{self._content_id}/manifest.json",
            "GET",
        )
        return resp

    async def get_content_item(self, name):
        if not self._content_id:
            raise SpecificationError(
                "content_id", "Content not uploaded for inspection"
            )
        return await self.accqsure._query(
            f"/inspection/{self.id}/asset/{self._content_id}/{name}",
            "GET",
        )



import logging


class AccQsureException(Exception):
    def __init__(self, message, *args):
        super().__init__(message, *args)
        self._message = message

    @property
    def message(self) -> str:
        return self._message

    def __repr__(self):
        return "AccQsureException( {self.message!r})".format(self=self)

    def __str__(self):
        return "AccQsureException({self.message!r})".format(self=self)


class ApiError(AccQsureException):
    def __init__(self, status, data, *args):
        super().__init__(data, *args)
        self._status = status
        logging.debug(data)
        self._message = data.get("errorMessage") or data.get("message")

    @property
    def status(self) -> int:
        return self._status

    def __repr__(self):
        return "ApiError({self.status}, {self.message!r})".format(self=self)

    def __str__(self):
        return "ApiError({self.status}, {self.message!r})".format(self=self)


class SpecificationError(AccQsureException):
    def __init__(self, attribute, message, *args):
        super().__init__(message, *args)
        self._attribute = attribute
        self._message = message

    @property
    def attribute(self) -> str:
        return self._attribute

    def __repr__(self):
        return "SpecificationError({self.attribute}, {self.message})".format(
            self=self
        )

    def __str__(self):
        return "SpecificationError({self.attribute}, {self.message})".format(
            self=self
        )


class TaskError(AccQsureException):
    def __init__(self, message, *args):
        super().__init__(message, *args)
        self._message = message

    def __repr__(self):
        return "TaskError({self.attribute}, {self.message})".format(self=self)

    def __str__(self):
        return "TaskError({self.attribute}, {self.message})".format(self=self)



import click
import os
import base64
import magic
from enum import Enum
from accqsure.cli import cli, pass_config


class FILE_FORMATS(str, Enum):
    DOCX = "docx"
    TEXT = "text"
    XLSX = "xlsx"
    CSV = "csv"
    PDF = "pdf"


DOCUMENT_TYPES = {
    "application/vnd.openxmlformats-officedocument.wordprocessingml.document": FILE_FORMATS.DOCX,
    "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet": FILE_FORMATS.XLSX,
    "text/plain": FILE_FORMATS.TEXT,
    "application/json": FILE_FORMATS.TEXT,
    "text/csv": FILE_FORMATS.CSV,
    "text/markdown": FILE_FORMATS.TEXT,
    "application/pdf": FILE_FORMATS.PDF,
}


@cli.group()
@pass_config
def inspection(config):
    """AccQsure inspection commands."""
    pass


@inspection.command()
@click.argument("type", type=click.STRING)
@pass_config
def list(config, type):
    """List inspections."""
    data = [
        ["ID", "NAME", "STATUS"],
        [
            "-" * 80,
            "-" * 80,
            "-" * 80,
        ],
    ]

    inspections = config.accqsure.run(
        config.accqsure.client.inspections.list(type=type)
    )
    for doc in inspections:
        data.append(
            [
                doc.id,
                doc.name,
                doc.status,
            ]
        )
    for row in data:
        click.echo(
            "{: >26.24} {: >40.38} {: >14.12} " "".format(*row),
            file=config.stdout,
        )


# @inspection.command()
# @click.option("--type", "-t", type=click.STRING, required=True)
# @click.option(
#     "--file",
#     "-f",
#     type=click.Path(
#         exists=True, file_okay=True, dir_okay=False, resolve_path=True
#     ),
#     required=True,
# )
# @click.argument("name", type=click.STRING, required=True)
# @click.argument("doc-id", type=click.STRING, required=True)
# @pass_config
# def create(config, name, doc_id, type, file):
#     """
#     Create an inspection

#     """

#     mime = magic.Magic(mime=True)
#     mime_type = mime.from_file(file)

#     if mime_type not in DOCUMENT_TYPES:
#         raise ValueError(
#             f"Invalid file type. Detected MIME type '{mime_type}' not in allowed types: {', '.join(DOCUMENT_TYPES.values())}"
#         )

#     file_type = DOCUMENT_TYPES[mime_type]

#     with open(os.path.expanduser(file), "rb") as f:
#         value = f.read()
#         base64_contents = base64.b64encode(value).decode("utf-8")

#     title = os.path.splitext(os.path.basename(file))[0]
#     result = config.accqsure.run(
#         config.accqsure.client.documents.create(
#             document_type_id=document_type_id,
#             name=name,
#             doc_id=doc_id,
#             contents=dict(
#                 title=title, type=file_type, base64_contents=base64_contents
#             ),
#         )
#     )
#     click.echo(result)
